#!/usr/bin/perl
#
# Script: textfilter.pl
#
# Purpose: This filter script converts plain text files
#          into an HTML document but keeps the text layout
#          as is.
#
# Copyright (c) 2001 by Mohammed J. Kabir
#
# License: GPL
#
# Listing 5-1: textfilter.pl
#
# The copyright message file is always stored in
# the server's document root
# directory and is called copyright.html.
#
my $copyright_file = $ENV{DOCUMENT_ROOT} . "/copyright.html";
# Get the requested document's path
my $path_translated = $ENV{PATH_TRANSLATED};

# Other variables needed for storing data
my $line;
my @text;
my @html;

# Store the path info and the file name of requested doc in an array
@filename = split(/\//,$path_translated);

# Because HTML tags are used to display the text file,
# lets print the text/html content header.
print "Content-type: text/html\n\n";

# Read the document requested and store the data
# in @text array variable
@text = &readFile($path_translated);

# Now print the following HTML document tags.
# These tags will be sent before the actual document content
#
print <<HEAD;
   <HTML>
   <HEAD> <TITLE>$filename[-1] </TITLE> </HEAD>
   <BODY BGCOLOR="white">
   <BLOCKQUOTE>
   <PRE>
HEAD

# Now print each line stored in the @text array
# (that is, the content of the document requested)
#
foreach $line (@text) { print $line; }
# Now read the copyright file and store the content
# in the @html array variable
#
@html = &readFile($copyright_file);
# Print each line stored in the @html array (that is,
# the content of the copyright message file)
#
foreach $line (@html){ print $line; }
# Exit the filter
exit 0;

sub readFile {
   #
   # Subroutine: readFile
   # Purpose: Reads a file if it exists or else prints
   # an error message and exits script
   #

   # Get the name of the passed file name and store
   # it in variable $file
    my $file = shift;

   # Local buffer variable
    my @buffer;

   # If the file exists, open it and read all the
   # lines into the @buffer array variable
    if(-e $file) {

       open(FP,$file) || die "Can not open $file.";
       while(<FP>){
         push(@buffer,$_);
       }

       close(FP);
    } else {
        push(@buffer,"$file is missing.");
    }

   # Return the content of the buffer.
   return (@buffer);
}
