#!/usr/bin/perl
#
# Written by Mohammed J. Kabir (mjkabir@mobidac.com
#
#
# Copyright (c) 2001 by Mohammed J. Kabir
#
# License: GNU Public License
#
# Updates welcome.
#
############################################################

use Getopt::Long;
use HTTPD::UserAdmin;
use HTTPD::GroupAdmin;

#
# Global variables
#
my (@SQL, $db, $action, $name, $group, $password, $auto);

#
# Get command-line options
#
GetOptions( 
	"dbpwd=s" 	=> \$dbpwd,		
	"db=s" 		=> \$db,		
	"action=s" 	=> \$action,		
	"user=s" 	=> \$name,		
	"group=s" 	=> \$group,		
	"password=s" 	=> \$password,
	"auto=s" 	=> \$auto
	);

#
# User configrable section
#
my $DB_HOST 		= 'localhost';
my $DB_PORT 		= '';
my $DATABASE 		= 'auth';
my $DB_DRIVER 		= 'mysql';
my $DB_USER 		= 'kabir';
my $DB_PASSWORD 	= $dbpwd;
my $ENCRYPTION 		= 'crypt';    # Encryption:  crypt | none | MD5
my $USER_TABLE 		= 'wwwusers';
my $USERNAME_FIELD 	= 'username',
my $PASSWORD_FIELD 	= 'passwd',
my $GROUP_TABLE 	= 'wwwgroups';
my $GROUP_FIELD 	= 'groupname';
my $MAXSZ_USER 		= 40;
my $MAXSZ_PWD 		= 20;
my $MAXSZ_GRP 		= 40;

#
# End of user configurable section
#

#
# Find out which table we should be working on
#
$db = ($db ne '') ? lc $db : 'user';


# 
# Setup the SQL database related information
#
 @SQL =  (DBType 	=> "SQL",
         Host 	 	=> $DB_HOST,             
         Port 		=> $DB_PORT,                      
         DB 		=> $DATABASE,          
         User 		=> $DB_USER,             
         Auth 		=> $DB_PASSWORD,             
         Encrypt 	=> $ENCRYPTION,        
         Driver 	=> $DB_DRIVER,         
         Server 	=> 'apache',
         UserTable 	=> $USER_TABLE,
         NameField 	=> $USERNAME_FIELD,
         GroupTable 	=> $GROUP_TABLE,
         GroupField 	=> $GROUP_FIELD,
         PasswordField 	=> $PASSWORD_FIELD
         );



#
# Lower case the action selected [add | delete | update | list ]
#
$action = lc $action;

# 
# Create UserAdmin and GroupAdmin objects
#
my $user = new HTTPD::UserAdmin @SQL;
my $grp = new HTTPD::GroupAdmin @SQL;

#
# If -db=user then we want to work on the user table
#
if ($db =~ /^u/ ) {

    # Add new user
    add_user($name, $password) 	 if ($action =~ /^a/);		

    # Delete existing user
    remove_user($name) 		 if ($action =~ /^d/);

    # Update existing user password
    update_user($name,$password) if ($action =~ /^u/);

    # List all users
    list_users() 		 if ($action =~ /^l/);

    # Delete all users
    delete_all_users() 		 if ($action =~ /^empty_table/);

#
# Since -db=!user = group we want to work on group table
#
} else {

   #`
   # Group operations require a valid user 
   # If user does not exist, abort
   if (! $user->exists($name)) {
      print "User $name does not exist!\n";
      exit 0;
   }

   # Add user to a group. Currently only way to add
   # add a new group is to add a user first.
   add_user_to_group($name, $group) 	 if ($action =~/^a/);

   # Remove a user from named group
   remove_user_from_group($name, $group) if ($action =~/^d/);

}

exit 0;


# ---------------------------------------------------#
# GroupAdmin routines                                #
# ---------------------------------------------------#
sub add_user_to_group {

   my ($u, $g) = @_;
   $g = lc $g;

   if ($grp->add($u, $g)) {
       print "User $u is now a member of the $group group.\n";
   } else {
      print "User $u could not be added to the $group group!\n";
   }
       
}

sub remove_user_from_group {

   my ($u, $g) = @_;
   $g = lc $g;
   if ($grp->exists($g)) {
       $grp->delete($u,$g);
   } else {
       print "Group $g was not found!\n";
   }

}

# ---------------------------------------------------#
# UserAdmin routines                                 #
# ---------------------------------------------------#

sub delete_all_users {

    print "Do you really want to delete all the users?\n";
    print "Press control+c to abort or Enter to continue.\n";

    my $ans = <STDIN>;
    sleep 3;

    my $count = 0;
    foreach my $u ($user->list()) {
       print "Deleting $u\n";
       remove_user($u);
       $count++;
    }

    print "Total $count user(s) deleted.\n";
}

sub list_users {

    foreach my $u ($user->list()) {
       print "$u\n";
    }
}

sub add_user {
    $u = lc shift;
    $p = shift;

    if ($p eq '' || length($p) >= $MAXSZ_PWD) {
        print "Password cannot exceed $MAXSZ_PWD characters.\n"; 
    } elsif ($user->exists($u)) {
        print "User $u already exists!\n";
    } else{
        if (length($u) <= $MAXSZ_USER && ($user->add($u, $p))) {
           print "User $u added.\n";
        } else {
           print "User $u is too long. Maximum size is $MAXSZ_USER characters.\n";
        }
    }
}

sub update_user {
    $u = lc shift;
    $p = shift;

    if ($p eq '' || length($p) >= $MAXSZ_PWD) {
        print "Password cannot exceed $MAXSZ_PWD characters.\n"; 
    } elsif (! $user->exists($u)) {
        print "User $u does not exist!\n";
    } elsif ($user->update($u, $p)) {
        print "User $u updated.\n";
    } else {
        print "User $u could not be updated to the database.\n";
    }
}

sub remove_user {
    $u = lc shift;

    if (! $user->exists($u)) {
        print "User $u does not exist!\n";
    } elsif ($user->delete($u)) {

        if ($auto) {
            foreach my $g ($grp->list) {
              remove_user_from_group($u, $g) if ($grp->exists($g, $u));
            }
        }

        print "User $u deleted.\n";
    } else {
        print "User $u could not be deleted from the database.\n";
    }
}
