#!/usr/bin/perl

# Version 1.0.1

use strict;

# Make sure you change the following path based on where you keep
# the HTML::Template packages. If you install HTML::Template for
# the entire system then you do not need to worry about this path
# Else point this path to the directory where you keep your
# Perl packages such as HTML::Template.
use lib qw(/www/cgi-bin);

use HTML::Template;
use File::Basename;
use constant DEBUG => 1;

# Make sure you change this path to your Web site's top directory
use constant FQPN_DOC_ROOT => "/home/mjkabir/www";

my $DEFAULT_TEMPLATE_FILE = sprintf("%s/default-tmpl.html", FQPN_DOC_ROOT);
my $DEFAULT_RSB_FILE = sprintf("%s/default-rsb.html", FQPN_DOC_ROOT);
my $DEFAULT_LSB_FILE = sprintf("%s/default-lsb.html", FQPN_DOC_ROOT);

my $DEFAULT_TOP_FILE = sprintf("%s/default-top.html", FQPN_DOC_ROOT);
my $DEFAULT_BOTTOM_FILE = sprintf("%s/default-bottom.html", FQPN_DOC_ROOT);



foreach my $file (@ARGV) {
   
   next if ($file !~ /\.txt$/);
   DEBUG and print "Processing $file \n";
   
   my $dir  = dirname($file);
   my ($base) = split(/\./, basename($file));

   process_file($base, $dir, $file);
}


sub get_sidebar_filename {
    my ($base, $dir, $type) = @_;

    my $file = sprintf("%s/%s.%s.html",$dir, $base, $type);
    
    my $dirName = $dir;
    $dirName =~ s/\/$//;
    $dirName = basename($dirName);

    # if we cannot find $base specific sidebar file then
    # try the $dir level sidebar file then the default top-level one
    $file = sprintf("%s/%s-%s.html",$dir, $dirName, $type) if ( ! -e $file );

    printf("%s template %s ", uc($type), $file);
     
    $file = ($type eq 'rsb') ? $DEFAULT_RSB_FILE : $DEFAULT_LSB_FILE if (! -e $file);

    printf("chosen: %s\n", $file);

    return $file;
}

sub get_updown_nav_filename {
    my ($base, $dir, $type) = @_;

    my $file = sprintf("%s/%s.%s.html",$dir, $base, $type);
    
    my $dirName = $dir;
    $dirName =~ s/\/$//;
    $dirName = basename($dirName);

    # if we cannot find $base specific sidebar file then
    # try the $dir level sidebar file then the default top-level one
    $file = sprintf("%s/%s-%s.html",$dir, $dirName, $type) if ( ! -e $file );
     
    $file = ($type eq 'bottom') ? $DEFAULT_BOTTOM_FILE : $DEFAULT_TOP_FILE if (! -e $file);

    return $file;
}

sub get_template_filename {
    my ($base, $dir) = @_;

    my $dirName = $dir;
    $dirName =~ s/\/$//;
    $dirName = basename($dirName);

    my $file = sprintf("%s/%s.tmpl.html",$dir, $base);

    $file = sprintf("%s/%s-tmpl.html",$dir, $dirName) if ( ! -e $file );

    $file = $DEFAULT_TEMPLATE_FILE if (! -e $file);

    return $file;
}
        
sub process_file {
   my ($base, $dir, $file)  = @_;
   
   my $rsbFile      = get_sidebar_filename($base, $dir, 'rsb');          
   my $lsbFile      = get_sidebar_filename($base, $dir, 'lsb');      
   my $bottomFile   = get_updown_nav_filename($base, $dir, 'bottom');
   my $topFile      = get_updown_nav_filename($base, $dir, 'top');
   
   my $templateFile = get_template_filename($base, $dir);
         
   my $bodyFile = sprintf("%s/%s.txt",$dir, $base);
   
   if ( ! -e $bodyFile ) {
       print "Cannot find body file: $bodyFile! Skipping!\n";
       return 0;
   }        
   
   my $htmlFile       = $dir . '/' . $base . '.html';
   my $htmlBackupFile = $dir . '/' . $base . '.html.bak';
   
   if (-e $htmlFile ) {
      rename $htmlFile, $htmlBackupFile;
      DEBUG and print "Backed up $htmlFile as $htmlBackupFile\n";
   }

   DEBUG and print "       Template: $templateFile\n" .
                   "      BODY file: $bodyFile \n" . 
                   "       LSB file: $lsbFile\n" .
                   "       RSB file: $rsbFile\n" .
                   "Bottom Nav file: $bottomFile\n" .
                   "   Top Nav file: $topFile\n" .
                   "      HTML file: $htmlFile\n";
   
      
   
   my $template = new HTML::Template(
                        filename => $templateFile,
                        die_on_bad_params => 0
                        );
   
   my $body = get_contents($bodyFile);
   my $sectionName = ($body =~ /<!--\s*section=(.*)\s*-->/) ? $1 : '';
   my $sectionURL = ($body =~ /<!--\s*sectionURL=(.*)\s*-->/) ? $1 : '';
   my $sectionTitle = ($body =~ /<!--\s*sectionTitle=(.*)\s*-->/) ? $1 : '';
   my $pageTitle = ($body =~ /<!--\s*pageTitle=(.*)\s*-->/) ? $1 : '';
   my $h1 = ($body =~ /<!--\s*H1=(.*)\s*-->/) ? $1 : '';
   my $h2 = ($body =~ /<!--\s*H2=(.*)\s*-->/) ? $1 : '';
   my $scriptArgs = ($body =~ /<!--\s*ARGS=(.*)\s*-->/) ? $1 : '';
   my $rsb = get_contents($rsbFile);
   my $lsb = get_contents($lsbFile);
   my $bottomMenu = get_contents($bottomFile);
   my $topMenu = get_contents($topFile);
   my $lastUpdated = localtime();
   # fill in some parameters
   $template->param(
   BODY => $body,
   RIGHT_SIDE_BAR => $rsb,
   LEFT_SIDE_BAR => $lsb,
   BOTTOM_NAVIGATION_MENU => $bottomMenu,
   SECTION_NAME => $sectionName,
   SECTION_URL => $sectionURL,
   SECTION_TITLE => $sectionTitle,
   PAGE_TITLE => $pageTitle,
   HEADLINE1=> $h1,
   HEADLINE2=> $h2,
   LAST_UPDATED => $lastUpdated,
   SCRIPT_ARGS => $scriptArgs,
   ID => $$
   );
   
   # print the template
   my $html = $template->output;
   
   
   
   
   open(HTML, "> $htmlFile") || die "Can't write $htmlFile $!\n";
   print HTML $html;
   close(HTML);

}


sub get_contents {
  
   my $file = shift;

   local $/ = undef;
   open(FILE, $file) || return 0;
   my $data = <FILE>;
   close(FILE);

   return $data;
}
