#!/usr/bin/perl -w
#
#
#  Name: mime-mail.pl
#
#  Purpose:
#
#  Sends the referrer page via email
#
####################################################

use strict;

#use CGI;
use CGI qw/:standard -debug/;

use MIME::Lite;
use MIME::Lite::HTML;
use Config::IniFiles;
use File::Basename;

#
# Create a CGI object
#
my $query = new CGI;

#
# Create a configuration object by loading the mime-mail.ini
# file from the physical directory pointed by /cgi-bin/
#
my $cfg      = new Config::IniFiles -file => "mime-mail.ini";


#
# Get the referrer URL
#
my $referrerURL = $ENV{HTTP_REFERER};

#
# Get the referrer page name from the referrer
# information
my $pageName = get_form_name($referrerURL);

#
# Print content type header to be text/html
#
print $query->header;

#
# If the referrer page does not have a [section] in
# the mime-mail.ini file than use default section
$pageName = 'defaults' if ($cfg->val($pageName, 'from') eq '');

#
# Get the email address. If no email address is supplied,
# show error message.
#
my $to = ( $query->param('email') eq '' || 
           ! check_email($query->param('email'), 'email')) ?
           abort('Email address is missing.') : $query->param('email');

my $senderName = ( $query->param('name') eq '') ?
           abort('Your name is missing.') : $query->param('name');

$senderName = format_name($senderName);


#
# Create a MIME::Lite::HTML object and initialize
# it with referrer, an
my $mailHTML = new MIME::Lite::HTML
                   From => $senderName . '<' .
                           $cfg->val($pageName, 'from') . '>',
                   To   => $to,
                   Subject => $cfg->val($pageName, 'subject');


#
# Parse the referrer URL and create a MIME mail object
#
my $MIMEmail = $mailHTML->parse($referrerURL);

#
# Send the MIME mail
#
$MIMEmail->send;

#
# Check error string for errors
#
my @errors = $mailHTML->errstr;


#
# If no error found, tell the user that page was
# sent to her friend. Else show error message
#
if ($#errors > -1 ) {
    print abort('Error(s) found: ' . join('\\n', @errors));
} else {
    print abort("Page sent to $to.\\nThank you.");
}


#
# Terminate
#
exit 0;


sub abort {

   # Print javascript abort message
   print sprintf("<script>alert('%s');history.go(-1);</script>",shift);
   exit 0;
}

sub get_form_name {
   my $referrer = shift;

   # If the HTTP referrer has a query string in it
   # return only the non-query string part of it
   #
   if ($referrer =~ /\?/) {

       ($referrer, undef) = split(/\?/,$referrer);

   }

   return basename($referrer);
}

sub format_name {

    my @str = ();
    foreach my $part (split(/\s/,shift)) {
        push(@str, ucfirst(lc($part)));
    }

    return join(' ',@str);
}        

sub check_email {
   my $email = shift;
   my $fieldName = shift;
   
   # Special note
   # This email cheacker routine might return false (0)
   # for some rarely used RFC-compliant email addresses
   # that are not common. For example bob&sandra@domain.com
   # will fail. If you wish to allow such "technically" valid
   # emails, you must modify the regular expressions used
   # in this routine. 
   
   # Untaint email address
   $email =~ /(\S+)\@([\w.-]+)/;
   $email =  $1 . '@' . $2;
   
   # Store untainted value in query object
   $query->param($fieldName, $email);
      
   # Split the user@host email into user and host
   my ($user, $host) = split(/@/, $email);
   
   # Return false if host part does not have
   # hostname.domain.tld or domain.tld format
   #
   return ($host !~ /(^[a-z0-9._-]+\.[a-z]{2,3})$/i) ? 0 : 1;
}
