#!/bin/sh
#
# This script runs rdist to update Web servers via the
# non-routable lan domain.com. The script is run
# by cron at a fixed interval.
#
# /etc/rc.d/rc.local starts the script to clean up
# left-over tempfiles that might have been left
# at shutdown. This process also removes the
# log file.
#
# $Author$ (kabir@evoknow.com)
# $Version$
# $Id$
# $Date$
# $Status
#
# Listing 23-3:	rdistribute.sh script
##############################################################

RDIST=/usr/bin/rdist
RDISTD=/usr/sbin/rdistd
DIST_FILE=/usr/local/rdist/rdist_distfile
RDIST_OPTIONS=remove,nochkgroup,nochkmode,nochkowner,quiet
RDIST_LOCK_FILE=/tmp/rdist.lck
RDIST_LOG_FILE=/tmp/rdist.log
TOUCH_BIN=/bin/touch
DATE=`date`

# If the script is called with an argument, then
case "$1" in
  boot)
  # Because the argument is 'boot,' the script is being
  # called at system start-up, so remove all old lock
  # files and logs.
  echo -n "Cleaning up rdistribute.sh tmp files: "
     rm -f $RDIST_LOCK_FILE
        rm -f $RDIST_LOG_FILE
  echo "complete."
  exit 0;
  ;;

  # Because the argument is 'restart,' the script
  # needs to clean up as if the system just booted.
  restart)
  $0 boot
  ;;

esac

# If the lock file exists, then don't do anything.
if [ -f $RDIST_LOCK_FILE ]; then
   exit 0
fi

# Otherwise, create the lock file using touch
$TOUCH_BIN $RDIST_LOCK_FILE

# Run rdist
$RDIST -p $RDISTD -o$RDIST_OPTIONS -f $DIST_FILE

# Remote the lock file
rm -f $RDIST_LOCK_FILE

# Write the time and date in the log file
echo $DATE >> $RDIST_LOG_FILE

# Exit the script
exit 0
